@@ -9,7 +9,7 @@ from sql.aggregate import Min, Max, Count, Sum
 from sql.conditionals import Coalesce, Case
 from sql.functions import Extract
 
-from trytond.model import ModelView, ModelSQL, Workflow, fields
+from trytond.model import ModelView, ModelSQL, Workflow, fields, Check
 from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond import backend
 from trytond.pyson import Equal, Eval, Not, In, If, Get, PYSONEncoder
@@ -143,9 +143,10 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
     def __setup__(cls):
         super(SaleOpportunity, cls).__setup__()
         cls._order.insert(0, ('start_date', 'DESC'))
+        t = cls.__table__()
         cls._sql_constraints += [
             ('check_percentage',
-                'CHECK(probability >= 0 AND probability <= 100)',
+                Check(t, (t.probability >= 0) & (t.probability <= 100)),
                 'Probability must be between 0 and 100.')
             ]
         cls._error_messages.update({
@@ -340,6 +341,7 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
         Date = pool.get('ir.date')
         cls.write(filter(lambda o: o.is_forecast, opportunities), {
                 'end_date': Date.today(),
+                'state': 'won',
                 })
 
     @classmethod
@@ -349,6 +351,7 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
         Date = Pool().get('ir.date')
         cls.write(filter(lambda o: o.is_forecast, opportunities), {
                 'end_date': Date.today(),
+                'state': 'lost',
                 })
 
     @classmethod
@@ -403,7 +406,9 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
         lost = []
         converted = []
         for opportunity in opportunities:
-            opportunity.amount = opportunity.sale_amount
+            sale_amount = opportunity.sale_amount
+            if opportunity.amount != sale_amount:
+                opportunity.amount = sale_amount
             if opportunity.is_won():
                 won.append(opportunity)
             elif opportunity.is_lost():
@@ -454,7 +459,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
     @staticmethod
     def order_sequence(tables):
         table, _ = tables[None]
-        return [table.sequence == Null, table.sequence]
+        return [Case((table.sequence == Null, 0), else_=1), table.sequence]
 
     @fields.depends('unit')
     def on_change_with_unit_digits(self, name=None):
@@ -504,10 +509,7 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
     employee = fields.Many2One('company.employee', 'Employee',
             datetime_field='date')
     start_date = fields.Date('Start Date')
-    end_date = fields.Date('End Date', states={
-        'invisible': Not(In(Eval('state'),
-            ['converted', 'cancelled', 'lost'])),
-    }, depends=['state'])
+    end_date = fields.Date('End Date')
     description = fields.Char('Description')
     comment = fields.Text('Comment')
     lines = fields.Function(fields.One2Many('sale.opportunity.line', None,
@@ -584,6 +586,7 @@ class SaleOpportunityHistory(ModelSQL, ModelView):
                         *time.strptime(values['date'],
                             '%Y-%m-%d %H:%M:%S.%f')[:6])
                 values['date'] = values['date'].replace(microsecond=0)
+                values['date'] += datetime.timedelta(seconds=1)
         return res
 
 
